/*********************************************************/
/* trans-code.c						 */
/* Copyright : Norwegian Telecom Research, 1993		 */
/* Programming : Jon Stephenson von Tetzchner		 */
/* This file can be changed and modified as long as this */
/* header is kept intact. This code can not be included	 */
/* in a commercial product.				 */
/*********************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include "trans.h"

#define FALSE 0     		/* Various useful constants */
#define TRUE 1
#define True 1
#define False 0

#define UNKNOWN 	0	/* Paragraph types */

#define HEADING0 	1	/* Headers */
#define HEADING1 	2
#define HEADING2 	3
#define HEADING3 	4
#define HEADING4 	5
#define HEADING5 	6
#define HEADING6 	7

#define PROGRAM  	9
#define REFERENCE 	10
#define TITLE           11
#define TOC		12
#define ADDRESS		13
#define AUTHOR	 	14
#define ABSTRACT 	15
#define LOCALTITLE	16
#define BODY     	17

#define COMREFERENCE 18
#define ACRONYM		20 

#define TABLE	    	23

#define FIGURE		25


#define LISTINDENT1	26	/* Non-consecutive lists with full indent */
#define LISTINDENT2	27	/* (HTML: <DL><DD>item) */
#define LISTINDENT3	28
#define LISTINDENT4	29
#define LISTINDENT5	30
#define LISTINDENT6	31

#define LISTBULLET1	32	/* Non-consecutive lists with bullet as mark */
#define LISTBULLET2	33	/* (HTML: <UL><LI>item) */
#define LISTBULLET3	34
#define LISTBULLET4	35
#define LISTBULLET5	36
#define LISTBULLET6	37

#define LISTMARK1 	38	/* Lists using MIF PgfNumString as mark */
#define LISTMARK2 	39	/* (HTML: <DL><DT>mark<DD>item) */
#define LISTMARK3 	40
#define LISTMARK4 	41
#define LISTMARK5 	42
#define LISTMARK6 	44

#define LISTNUMBER1 	45	/* Numbered consecutive lists */
#define LISTNUMBER2 	46	/* (HTML: <OL><LI>item) */
#define LISTNUMBER3 	47
#define LISTNUMBER4 	48
#define LISTNUMBER5 	49
#define LISTNUMBER6 	50

#define LISTCONTINUE1 	51	/* Additional paragraphs in any type of list */
#define LISTCONTINUE2 	52
#define LISTCONTINUE3 	53
#define LISTCONTINUE4 	54
#define LISTCONTINUE5 	55
#define LISTCONTINUE6 	56

#define CELLHEADING	57
#define CELLBODY	58

#define UHEADING0 	59	/* Headers not appearing in the table of contents*/
#define UHEADING1 	60
#define UHEADING2 	61
#define UHEADING3 	62
#define UHEADING4 	63
#define UHEADING5 	64
#define UHEADING6 	65

#define INDENTHEADING0  69
#define INDENTHEADING1	70	/* Indented headers */
#define INDENTHEADING2	71
#define INDENTHEADING3	72
#define INDENTHEADING4	73
#define INDENTHEADING5	74
#define INDENTHEADING6	75

#define GLOSSARY   	80
#define GLOSSARY2	81
#define GLOSSARY3	82
#define COMGLOSSARY	83
#define COMGLOSSARY2	84
#define COMGLOSSARY3	85

#define UNORDERED	0	/* HTML list types */
#define ORDERED		1
#define DESCRIPTION	2

#define MAXDEPTH	6	/* Maximum list levels */
#define STACKSIZE	(MAXDEPTH + 1)

int depth = 0;		/* Current level of list nesting */
int tcdepth = 0;	/* Current level of table of contents nesting */
int stack[STACKSIZE];	/* Type of list found at each level of nesting */


#define MAXNUMFOOTNOTES 	100		
#define MAXNUMREFERENCES 	500


#ifndef kEM_DASH_PREF
#define kEM_DASH_PREF "---"
#endif

#ifndef kEN_DASH_PREF
#define kEN_DASH_PREF "--"
#endif

int in_string = FALSE;	/* Used to identify that inside a string. */
                        /* Thus following strings are added to a paragraph. */

int in_reference = FALSE;
int in_glossary = FALSE;
int in_program = FALSE;
int in_acronym = FALSE;
int in_table = FALSE;

#define NORMAL 0
#define SYMBOL 1
#define DINGBATS 2

int font_type = NORMAL;
int font_bold = FALSE;
int font_italic = FALSE;
int font_underline = FALSE;
int font_supscript = FALSE;
int font_subscript = FALSE;

int marker_number;			/* Paragraph marker ID */
int marker_type;			/* Marker type */
char *marker_text = NULL;		/* Marker text for hypertext markers */
int textrect;				/* Textrectid */
int header = -1;			/* Identifies paragraph type using */
                                        /* the types specified above */

int header_id = 0;			/* Used to discriminate header ids */

char *footnote[MAXNUMFOOTNOTES];	/* Holds all footnotes found which */
                                        /* are written in the end */
int footnote_def_nr = 0;		/* Assume that footnotes come one */
                                        /* after one in use and def */
int footnote_use_nr = 0;		
int in_footnote = FALSE;		

int table_num = 1;			/* Faar aa gjenkjenne figurer */
int figur_num = 1;
int inlineframe_num = 0;		/* Number of inline frames */

char *reference_filename;	/* Holds current reference filename */
int reference_number;		/* Holds reference id of current reference */

char *para_num_string;		/* Holds current paragraph numbering */

char *TRANSFilename;	       	/* Name of file to be created by trans */
                                /* shell script */
char *INDEXFilename;		/* Where to put index */
char *TITLEFilename;		/* Where to put title */
char *TAGSFilename;		/* File holding fm-tags to html-tags 
				   conversion info */

char pagefilename[200];
char textfilename[200];

FILE *pagefile;			/* File to hold page info */
FILE *textfile;			/* File to hold textflow */
FILE *f;			/* Points to one of the two above */
FILE *indexfile;		/* File to hold index */
FILE *titlefile;		/* File to hold title */
FILE *tagsfile;

char fm_tags[500][100];
int int_tags[500];
int tags_count = 0;

int tabfig[100];		/* Holds indicators of TABLEs and FIGUREs */
int tabfignum[100];		
int tabfig_num = 0;

char paragraph[50000];		/* Holds the current paragraph */

int textrect_count = 0;
int textrect_id[100];		/* TextRectIDs used in this frame */
int frame_id_arr[10000];	/* Keeping track of which frame (the value) a
				   given textrect (the index) is plased. */

int in_frame = FALSE;
int frame_id = -1;
FILE *framef;
char frame_filename[300];
char sys_call[10000];

#define INLINEFRAME	0
#define OTHERFRAME  	1

int num_frames = 0;
int frametype[1000][2];		/* FrameType[n][0] Frame id number */
                                /* FrameType[n][1] Frame type */
char *framename[1000];

char *NOGRAPHICS = NULL;    /* Whether to omit conversion of
                                contents of frames */
char *NOTOC = NULL; 	   /* Whether to omit generation of a
			       table of contents */


/*****************************************************************************
  int GetIntTag() :
 *****************************************************************************/

int GetIntTag( tag )
      char* tag;
{
    if (strcmp(tag,"UNKNOWN") == 0) return UNKNOWN;
    else if (strcmp(tag,"HEADING0") == 0) return HEADING0;
    else if (strcmp(tag,"HEADING1") == 0) return HEADING1;
    else if (strcmp(tag,"HEADING2") == 0) return HEADING2;
    else if (strcmp(tag,"HEADING3") == 0) return HEADING3;
    else if (strcmp(tag,"HEADING4") == 0) return HEADING4;
    else if (strcmp(tag,"HEADING5") == 0) return HEADING5;
    else if (strcmp(tag,"HEADING6") == 0) return HEADING6;
    else if (strcmp(tag,"UHEADING0") == 0) return UHEADING0;
    else if (strcmp(tag,"UHEADING1") == 0) return UHEADING1;
    else if (strcmp(tag,"UHEADING2") == 0) return UHEADING2;
    else if (strcmp(tag,"UHEADING3") == 0) return UHEADING3;
    else if (strcmp(tag,"UHEADING4") == 0) return UHEADING4;
    else if (strcmp(tag,"UHEADING5") == 0) return UHEADING5;
    else if (strcmp(tag,"UHEADING6") == 0) return UHEADING6;
    else if (strcmp(tag,"BODY") == 0) return BODY;
    else if (strcmp(tag,"PROGRAM") == 0) return PROGRAM;
    else if (strcmp(tag,"REFERENCE") == 0) return REFERENCE;
    else if (strcmp(tag,"COMREFERENCE") == 0) return COMREFERENCE;
    else if (strcmp(tag,"TITLE") == 0) return TITLE;
    else if (strcmp(tag,"LOCALTITLE") == 0) return LOCALTITLE;
    else if (strcmp(tag,"TOC") == 0) return TOC;
    else if (strcmp(tag,"ADDRESS") == 0) return ADDRESS;
    else if (strcmp(tag,"AUTHOR") == 0) return AUTHOR;
    else if (strcmp(tag,"ABSTRACT") == 0) return ABSTRACT;
    else if (strcmp(tag,"INDENTHEADING0") == 0) return INDENTHEADING0;
    else if (strcmp(tag,"INDENTHEADING1") == 0) return INDENTHEADING1;
    else if (strcmp(tag,"INDENTHEADING2") == 0) return INDENTHEADING2;
    else if (strcmp(tag,"INDENTHEADING3") == 0) return INDENTHEADING3;
    else if (strcmp(tag,"INDENTHEADING4") == 0) return INDENTHEADING4;
    else if (strcmp(tag,"INDENTHEADING5") == 0) return INDENTHEADING5;
    else if (strcmp(tag,"INDENTHEADING6") == 0) return INDENTHEADING6;
    else if (strcmp(tag,"TABLE") == 0) return TABLE;
    else if (strcmp(tag,"FIGURE") == 0) return FIGURE;
    else if (strcmp(tag,"GLOSSARY") == 0) return GLOSSARY;
    else if (strcmp(tag,"GLOSSARY2") == 0) return GLOSSARY2;
    else if (strcmp(tag,"GLOSSARY3") == 0) return GLOSSARY3;
    else if (strcmp(tag,"COMGLOSSARY") == 0) return COMGLOSSARY;
    else if (strcmp(tag,"COMGLOSSARY2") == 0) return COMGLOSSARY2;
    else if (strcmp(tag,"COMGLOSSARY3") == 0) return COMGLOSSARY3;
    else if (strcmp(tag,"ACRONYM") == 0) return ACRONYM;
    else if (strcmp(tag,"LISTINDENT1") == 0) return LISTINDENT1;
    else if (strcmp(tag,"LISTINDENT2") == 0) return LISTINDENT2;
    else if (strcmp(tag,"LISTINDENT3") == 0) return LISTINDENT3;
    else if (strcmp(tag,"LISTINDENT4") == 0) return LISTINDENT4;
    else if (strcmp(tag,"LISTINDENT5") == 0) return LISTINDENT5;
    else if (strcmp(tag,"LISTINDENT6") == 0) return LISTINDENT6;
    else if (strcmp(tag,"LISTBULLET1") == 0) return LISTBULLET1;
    else if (strcmp(tag,"LISTBULLET2") == 0) return LISTBULLET2;
    else if (strcmp(tag,"LISTBULLET3") == 0) return LISTBULLET3;
    else if (strcmp(tag,"LISTBULLET4") == 0) return LISTBULLET4;
    else if (strcmp(tag,"LISTBULLET5") == 0) return LISTBULLET5;
    else if (strcmp(tag,"LISTBULLET6") == 0) return LISTBULLET6;
    else if (strcmp(tag,"LISTMARK1") == 0) return LISTMARK1;
    else if (strcmp(tag,"LISTMARK2") == 0) return LISTMARK2;
    else if (strcmp(tag,"LISTMARK3") == 0) return LISTMARK3;
    else if (strcmp(tag,"LISTMARK4") == 0) return LISTMARK4;
    else if (strcmp(tag,"LISTMARK5") == 0) return LISTMARK5;
    else if (strcmp(tag,"LISTMARK6") == 0) return LISTMARK6;
    else if (strcmp(tag,"LISTNUMBER1") == 0) return LISTNUMBER1;
    else if (strcmp(tag,"LISTNUMBER2") == 0) return LISTNUMBER2;
    else if (strcmp(tag,"LISTNUMBER3") == 0) return LISTNUMBER3;
    else if (strcmp(tag,"LISTNUMBER4") == 0) return LISTNUMBER4;
    else if (strcmp(tag,"LISTNUMBER5") == 0) return LISTNUMBER5;
    else if (strcmp(tag,"LISTNUMBER6") == 0) return LISTNUMBER6;
    else if (strcmp(tag,"LISTCONTINUE1") == 0) return LISTCONTINUE1;
    else if (strcmp(tag,"LISTCONTINUE2") == 0) return LISTCONTINUE2;
    else if (strcmp(tag,"LISTCONTINUE3") == 0) return LISTCONTINUE3;
    else if (strcmp(tag,"LISTCONTINUE4") == 0) return LISTCONTINUE4;
    else if (strcmp(tag,"LISTCONTINUE5") == 0) return LISTCONTINUE5;
    else if (strcmp(tag,"LISTCONTINUE6") == 0) return LISTCONTINUE6;
    else if (strcmp(tag,"CELLHEADING") == 0) return CELLHEADING;
    else if (strcmp(tag,"CELLBODY") == 0) return CELLBODY;
    else 
	{
	    fprintf(stderr, "trans: Unknown internal tag: %s\n", tag);
	    return UNKNOWN;
	}
}



/*****************************************************************************
 int IsIndent()
 *****************************************************************************/

int IsIndent(tag)
    int tag;
{
    return tag >= LISTINDENT1 && tag <= LISTINDENT6 ? TRUE : FALSE;
}


/*****************************************************************************
 int IsBullet()
 *****************************************************************************/

int IsBullet(tag)
    int tag;
{
    return tag >= LISTBULLET1 && tag <= LISTBULLET6 ? TRUE : FALSE;
}


/*****************************************************************************
 int IsMark()
 *****************************************************************************/

int IsMark(tag)
    int tag;
{
    return tag >= LISTMARK1 && tag <= LISTMARK6 ? TRUE : FALSE;
}


/*****************************************************************************
 int IsNumber()
 *****************************************************************************/

int IsNumber(tag)
    int tag;
{
    return tag >= LISTNUMBER1 && tag <= LISTNUMBER6 ? TRUE : FALSE;
}


/*****************************************************************************
 int IsList()
 *****************************************************************************/

int IsList(tag)
    int tag;
{
    return IsIndent(tag) || IsBullet(tag) || IsMark(tag) || IsNumber(tag);
}


/*****************************************************************************
 int IsContinue()
 *****************************************************************************/

int IsContinue(tag)
    int tag;
{
    return tag >= LISTCONTINUE1 && tag <= LISTCONTINUE6 ? TRUE : FALSE;
}


/*****************************************************************************
 int Level() : return the nesting level associated with the given bullet or
	       list tag
 *****************************************************************************/

int Level(tag)
    int tag;
{
    if (IsIndent(tag))
	return tag + 1 - LISTINDENT1;
    else if (IsBullet(tag))
	return tag + 1 - LISTBULLET1;
    else if (IsMark(tag))
	return tag + 1 - LISTMARK1;
    else if (IsNumber(tag))
	return tag + 1 - LISTNUMBER1;
    else
	return 0;
}


/*****************************************************************************
 int ListType()
 *****************************************************************************/

int ListType(tag)
    int tag;
{
    return IsNumber(tag) ? ORDERED : IsBullet(tag) ? UNORDERED : DESCRIPTION;
}


/*****************************************************************************
 void TCDescend()
 *****************************************************************************/

void TCDescend(tclevels)
    int tclevels;
{
    if (!NOTOC)
	{
	    fprintf(indexfile, "\n");
	    for ( ; tclevels; tcdepth++, tclevels--)
		fprintf(indexfile, "<UL>\n");
	}
}


/*****************************************************************************
 void TCAscend()
 *****************************************************************************/

void TCAscend(tclevels)
    int tclevels;
{
    if (!NOTOC)
	{
	    if (tclevels == 0)
		fprintf(indexfile, "<BR>\n");
	    else
		{
		    for ( ; tclevels; tcdepth--, tclevels--)
			fprintf(indexfile, "</UL>\n");
		}
	    fprintf(indexfile, "\n");
	}
}


/*****************************************************************************
 void Descend()
 *****************************************************************************/

void Descend(list_type, levels)
    int list_type;
    int levels;
{
    fprintf(f, "\n");
    for ( ; levels; stack[++depth] = list_type, levels--)
	    fprintf(f, "%s\n", list_type == ORDERED ? "<OL>" :
		    list_type == UNORDERED ? "<UL>" : "<DL>");
}


/*****************************************************************************
 void Ascend()
 *****************************************************************************/

void Ascend(levels)
    int levels;
{
    for ( ; levels; depth--, levels--)
	fprintf(f, "%s\n", stack[depth] == ORDERED ? "</OL>\n" :
		stack[depth] == UNORDERED ? "</UL>\n" : "</DL>\n");
    fprintf(f, "\n");
}



/*******************************************************************************
 void PrintString() : Prints to standard output the contents of a MIF String
                      statement.
 Special cases	    : Footnotes, TOCs
 *******************************************************************************/


void PrintString()
{
    char *string_contents;

    if (!in_string)
	PrintHeaderTypeStart();
    in_string = TRUE;
	    
    string_contents = GetContents();
    if (!string_contents) return;    

    if (in_footnote)
	{
	    if (footnote_def_nr >= MAXNUMFOOTNOTES)
		{
		    fprintf(stderr, "\ntrans: ERROR: Too many footnotes.\n\n");
		    return;
		}
	    if (marker_text)
		{
		    strcat(footnote[footnote_def_nr], marker_text);
		    strcat(footnote[footnote_def_nr], string_contents);
		    strcat(footnote[footnote_def_nr], "</A>");
		    free(marker_text);
		    marker_text = NULL;
		}	
	    else
		strcat(footnote[footnote_def_nr], string_contents);
	    free(string_contents);
	}		    
    else if (header == TOC) 
	{
	    /* REMOVE TABLE OF CONTENTS */
	    free(string_contents);
	}
    else
	{
	    if (marker_text)
		{
		    strcat(paragraph, marker_text);
		    strcat(paragraph, string_contents);
		    strcat(paragraph, "</A>");
		    free(marker_text);
		    marker_text = NULL;
		}
	    else
		strcat(paragraph, string_contents);
	    free(string_contents);
	}
}


/*******************************************************************************
 void SetPgfTag() : 
             Output : Set the current paragraph tag.
 *******************************************************************************/

void SetPgfTag()
{
    header = HeaderType();
}


/*******************************************************************************
 void PrintHeaderTypeStart() : 
 *******************************************************************************/

void PrintHeaderTypeStart()
{
    if (header == TOC /* or others */)        /* ejr */
      ;

    if (in_reference && (header != REFERENCE && header != COMREFERENCE))
	{
	    fprintf(f, "</DL>\n");
	    in_reference = FALSE;
	}
    else if (in_glossary && (header != in_glossary))
	{
	    /* Close-up the nested glossary */

	    switch ( in_glossary)
		{
		  case GLOSSARY:
		  case COMGLOSSARY:
		    fprintf(f, "</DL>\n");
		    break;
		  case GLOSSARY2:
		  case COMGLOSSARY2:
		    fprintf(f, "</DL></UL>\n");
		    break;
		  case GLOSSARY3:
		  case COMGLOSSARY3:
		    fprintf(f, "</DL></UL></UL>\n");
		    break;
		    
		  default:
		    fprintf(stderr, "Don't know glossary state %d\n", in_glossary);
		}
	    in_glossary = FALSE;
	}
    else if (in_acronym && header != ACRONYM)
	{
	    fprintf(f, "</DL>\n");
	    in_acronym = FALSE;
	}
    else if (in_program && header != PROGRAM)
	{
	    fprintf(f, "</PRE>\n");
	    in_program = FALSE;
	}
}	


/*******************************************************************************
 void PrintHeaderTypeEnd() : 
 *******************************************************************************/

void PrintHeaderTypeEnd()
{
    char *marks, *marke;
    
    marks = calloc(100, sizeof(char));
    marke = calloc(10, sizeof(char));
    
    if (marker_number)
	{
	    sprintf(marks, "<A NAME=\"REF%d\">", marker_number);
	    strcpy(marke, "</A>");
	}
    
    if (in_table)
	{
	    char *formatted_paragraph;
	    formatted_paragraph = calloc(strlen(paragraph) + 200, sizeof(char));
	    if (header == TABLE || header == CELLHEADING)
		sprintf(formatted_paragraph, "%s<B>%s%s</B>%s", marks, para_num_string, paragraph, marke);
	    else
		sprintf(formatted_paragraph, "%s%s%s%s", marks, para_num_string, paragraph, marke);
	    UseString(formatted_paragraph);
	    free(formatted_paragraph);
	}
    else
	{
	    
	    if (depth > 0 && ! IsList(header) && ! IsContinue(header))
		{
		    Ascend(depth);
		}
	    
	    if (header == BODY || header == CELLBODY)
		{
		    fprintf(f, "%s%s%s%s<p>\n", marks, para_num_string, paragraph, marke);
		}
	    else if (header == CELLHEADING)
		{
		    fprintf(f, "%s<B>%s%s</B>%s<p>\n", marks, para_num_string, paragraph, marke);
		}
	    else if (IsList(header))
		{
		    int relative_depth = Level(header) - depth;
		    int list_type = ListType(header);
		    
		    if (relative_depth > 0)
			{
			    Descend(list_type, relative_depth);
			}
		    else
			{
			    Ascend(-(relative_depth));
			    
			    if (list_type != stack[depth])
				{
				    Ascend(1);
				    Descend(list_type, 1);
				}
			}
		    
		    fprintf(f, "%s%s%s%s%s<BR>\n", marks,
			    IsBullet(header) || IsNumber(header) ? "<LI>" : "<DD>",
			    IsMark(header) ? para_num_string : "",
			    paragraph, marke);
		}
	    else if (IsContinue(header))
		{
		    fprintf(f, "%s%s%s<BR>\n", marks, paragraph, marke);
		}
	    else if (header == AUTHOR)
		{
		    fprintf(titlefile, "<ADDRESS>%s%s%s%s</ADDRESS>\n", 
			    marks, para_num_string, paragraph, marke);
		}
	    else if (header == ADDRESS)
		{
		    fprintf(f, "<ADDRESS>%s%s%s%s</ADDRESS>\n", 
			    marks, para_num_string, paragraph, marke);
		}
	    else if (header == TITLE)
		{
		    char *overtitle = getenv("TITLE");
		    if (!(overtitle && overtitle[0])) 
			{
			    char *tmptitle = GetTextOnly(paragraph);
			    fprintf(titlefile, "<TITLE>%s%s</TITLE>\n", 
				    para_num_string, tmptitle);
			    fprintf(titlefile, "<H1>%s%s%s%s</H1>\n", 
				    marks, para_num_string, paragraph, marke);
			    free(tmptitle);
			}
		}
	    else if (header == LOCALTITLE)
		{
		    char *tmptitle = GetTextOnly(paragraph);
		    fprintf(f, "<TITLE>%s%s</TITLE>\n", 
			    para_num_string, tmptitle);
		    /* fprintf(f, "<H1>%s%s%s%s</H1>\n", 
			    marks, para_num_string, paragraph, marke); */
		    free(tmptitle);
		    if (marker_number)
			{
			    fprintf(f, "<H1><A NAME=\"REF%d\">%s%s</A></H1>\n", 
				    marker_number, para_num_string, paragraph);
			    if (!NOTOC)
				{
				    fprintf(indexfile, "<A HREF=\"%s.html#REF%d\"><B>%s%s</B></A>\n", 
					    TRANSFilename, marker_number, 
					    para_num_string, paragraph);
				}
			}
		    else
			{
			    fprintf(f, "<H1><A NAME=\"HDT%d\">%s%s</A></H1>\n", 
				    header_id,
				    para_num_string, paragraph);
			    if (!NOTOC)
				{
				    fprintf(indexfile, "<A HREF=\"%s.html#HDT%d\"><B>%s%s</B></A>\n", 
					    TRANSFilename, header_id,
					    para_num_string, paragraph);
				}
			    header_id++;
			}
		}

	    else if (header >= HEADING0 && header <= HEADING6) 	
		{
		    int relative_depth = header - HEADING1 + 1 - tcdepth;
		    int hlevel = header - HEADING1 + 2;
		    char *bstart = calloc(4, sizeof(char));
		    char *bend = calloc(5, sizeof(char));
		    if (hlevel > 6) hlevel = 6;
		    if (hlevel <= 2)
			{
			    strcpy(bstart, "<B>");
			    strcpy(bend, "</B>");
			}
		    if (relative_depth > 0)
			{
			    TCDescend(relative_depth);
			}
		    else
			{
			    TCAscend(-(relative_depth));
			}
		    
		    if (marker_number)
			{
			    fprintf(f, "<H%d><A NAME=\"REF%d\">%s%s</A></H%d>\n", 
				    hlevel, marker_number, para_num_string, 
				    paragraph, hlevel);
			    if (!NOTOC)
				{
				    fprintf(indexfile, "<A HREF=\"%s.html#REF%d\">%s%s%s%s</A>\n", 
					    TRANSFilename, marker_number, 
					    bstart, para_num_string, paragraph, bend);
				}
			}
		    else
			{
			    fprintf(f, "<H%d><A NAME=\"HDR%d\">%s%s</A></H%d>\n", 
				    hlevel, header_id,
				    para_num_string, paragraph, hlevel);
			    if (!NOTOC)
				{
				    fprintf(indexfile, "<A HREF=\"%s.html#HDR%d\">%s%s%s%s</A>\n", 
					    TRANSFilename, header_id,
					    bstart, para_num_string, paragraph, bend);
				}
			    header_id++;
			}
		}
	    else if (header >= UHEADING0 && header <= UHEADING6) 
		{
		    int hlevel = header - UHEADING1 + 2;
		    if (hlevel > 6) hlevel = 6;

		    fprintf(f, "<H%d>%s%s%s%s</H%d>\n", 
			    hlevel, marks, para_num_string, paragraph, marke, hlevel);
		}
	    else if (header == TABLE || header == FIGURE) 
		{
		    fprintf(f, "<B>%s%s%s%s</B><p>\n", marks, para_num_string, paragraph, marke);
		}
	    else if (header == ABSTRACT)
		{
		    fprintf(f, "<EM>%s%s%s%s</EM>\n", marks, para_num_string, paragraph, marke);
		}    
	    else if (header >= INDENTHEADING0 && header <= INDENTHEADING6) 
		{
		    int hlevel = header - INDENTHEADING1 + 2;
		    if (hlevel > 6) hlevel = 6;

		    fprintf(f, "<UL><H%d>%s%s%s%s</H%d></UL>\n", 
			    hlevel, marks, para_num_string, paragraph, marke, hlevel);
		}
	    else if (header == REFERENCE || header == COMREFERENCE) 
		{
		    if (!in_reference)
			{
			    if (header == REFERENCE)
				fprintf(f, "<DL>\n");
			    else
				fprintf(f, "<DL COMPACT>\n");
			    in_reference = TRUE;
			}
		    fprintf(f, "<DT>%s%s%s<DD>%s\n", marks, para_num_string, marke, paragraph);     
		}
	    else if (header >= GLOSSARY &&  header <= COMGLOSSARY3) 
		{
		    if (!in_glossary)
			{
			    in_glossary = header;
			    if (header >= GLOSSARY && header <= GLOSSARY3)
				{
				    int i;
				    for (i = 1; i <= header - GLOSSARY;i++)
					fprintf(f, "<UL>");
				    fprintf(f, "<DL>\n");
				}
			    else
				{
				    int i;
				    for (i = 1; i <= header - COMGLOSSARY;i++)
					fprintf(f, "<UL>");
				    fprintf(f, "<DL COMPACT>\n");
				}
			}
		    fprintf(f, "<DT>%s%s%s%s\n", marks, para_num_string, paragraph, marke);
		}
	    else if (header == PROGRAM)
		{
		    if (!in_program)
			{
			    in_program = TRUE;
			    fprintf(f, "<PRE>\n");
			}
		    fprintf(f, "%s%s%s%s\n", marks, para_num_string, paragraph, marke);
		}
	    else if (header == ACRONYM) 
		{
		    if (!in_acronym)
			{
			    in_acronym = TRUE;
			    fprintf(f, "<DL COMPACT>\n");
			}
		    fprintf(f, "<DT>%s%s%s%s\n", marks, para_num_string, paragraph,
			    marke);
		}
           else if (header == TOC)     /* ejr */
               {                       /* Don't break flow */
               }

	    else 
		{
		    fprintf(f, "<UNKNOWN>%s%s%s%s</UNKNOWN><BR>\n", 
			    marks, para_num_string, paragraph, marke);
		}
	}	    
    paragraph[0] = 0;
    para_num_string[0] = 0;
    marker_number = 0;
    free(marks);
    free(marke);
}


/*******************************************************************************
 int HeaderType() : 
 *******************************************************************************/

int HeaderType()
{
    char *headert;
    int header_type;
    int i;

    headert = GetHdr();

    if (!headert) 
	{
	    header_type = UNKNOWN;
	    fprintf(stderr, "\n trans: UNKNOWN TAG: \"%s\"\n", headert); 
	}	    
    else {
	i = 0;
	while (i < tags_count && strcmp(fm_tags[i], headert)) i++;
	if (i < tags_count)
	    header_type = int_tags[i];
	else if (strstr(headert, "TOC") || strstr(headert, "IX") || strstr(headert, "LOT") ||
		 strstr(headert, "LOF") || strstr(headert, "AIX") || strstr(headert, "AML") ||
		 strstr(headert, "APL") || strstr(headert, "IOM") || strstr(headert, "LOM") ||
		 strstr(headert, "LOP") || strstr(headert, "SIX"))
	    header_type = TOC;
	else
	    {
		header_type = UNKNOWN;
		fprintf(stderr, "\n trans: UNKNOWN TAG: \"%s\"\n", headert); 
	    }
    }

    free(headert);
    return header_type;
}


/*******************************************************************************
 void StartPrint() : 
 *******************************************************************************/

void StartPrint()
{
    int j, k, c;
    char tag[100];
    char *overtitle = getenv("TITLE");
    int comment = FALSE;
    int found = FALSE;

    TRANSFilename = getenv("TRANSFilename");
    INDEXFilename = getenv("INDEXFilename");
    TITLEFilename = getenv("TITLEFilename");
    TAGSFilename  = getenv("TAGSFilename");
    NOGRAPHICS    = getenv("NOGRAPHICS");
    NOTOC	  = getenv("NOTOC");

    strcpy(pagefilename, TRANSFilename);
    strcat(pagefilename, ".tmp.000P");
    strcpy(textfilename, TRANSFilename);
    strcat(textfilename, ".tmp.000T");

    if (!(pagefile = fopen(pagefilename, "w")))
	{
	    fprintf(stderr, "trans: Could not open temporary file \"%s\"\n", pagefilename);
	    exit(-1);
	}

    if (!(textfile = fopen(textfilename, "w")))
	{
	    fprintf(stderr, "trans: Could not open temporary file \"%s\"\n", textfilename);
	    exit(-1);
	}

    f = pagefile;

    if (!NOTOC)
	{
	    if (!(indexfile = fopen(INDEXFilename, "a")))
		{
		    fprintf(stderr, "trans: Could not open temporary file \"%s\"\n", INDEXFilename);
		    exit(-1);
		}
	}

    if (!(titlefile = fopen(TITLEFilename, "a")))
	{
	    fprintf(stderr, "trans: Could not open temporary file \"%s\"\n", TITLEFilename);
	    exit(-1);
	}

    fprintf(stderr,"trans: Tagsfile is %s\n", TAGSFilename);
    if (!(tagsfile = fopen(TAGSFilename, "r")))
	{
	    fprintf(stderr, "trans: Could not open tags file \"%s\"\n", TAGSFilename);
	    exit(-1);
	}

    /* Fill tags array */
    
    j = 0;
    k = 0;

    while ((c=getc(tagsfile)) != EOF) 
	{
	if (c == '\t' && !found) 
	    {
		if (k > 0) 
		    {
			tag[k] = '\0';
			StripSpace(tag);
			k = 0;
			if (strlen(tag))
			    {	
				strcpy(fm_tags[tags_count],tag);
				found = TRUE;
			    }
		    }
	    } 
	else if (c == '\n' || c == '#' || c=='\t') 	
	    {
		if (k > 0)
		    {
			tag[k] = '\0';
			StripSpace(tag);
			k = 0;
			if (strlen(tag))
			    {
				int_tags[tags_count++] = GetIntTag(tag);
				found = FALSE;
			    }
		    }
		if (c == '#')
		    {
			comment = TRUE;
			found = FALSE;
		    }
		else if (c == '\n')
		    {
			comment = FALSE;
			found = FALSE;
		    }
	    } 
	else if (!comment)
	    {
		tag[k++] = c;
	    }
    }

/*    for (j=0; j < tags_count; j++)
	fprintf(stderr, "TAG::: \"%s\"\n", fm_tags[j]);
*/

    /* fprintf(f, "\n<HTML>\n"); */ 
    fprintf(f, "\n<!-- This file was created with the fm2html filter.\n");
    fprintf(f, "The filter is copyright Norwegian Telecom Research and \n");
    fprintf(f, "was programmed by Jon Stephenson von Tetzchner.  -->\n");
    para_num_string = malloc(100);
    reference_filename = malloc(100);
    paragraph[0] = 0;

    if (overtitle && overtitle[0]) 
	{
	    fprintf(pagefile, "<TITLE>%s</TITLE>\n", overtitle);
	}
}


/*******************************************************************************
 void EndPrint() : 
 *******************************************************************************/

void EndPrint()
{
  if (footnote_def_nr) PrintFootnotes();
/*  fprintf(pagefile, "\n</HTML>\n"); */

  if (tcdepth) TCAscend(tcdepth);
  if (!NOTOC)
      fprintf(indexfile, "\n");
  fprintf(textfile, "\n<<<\n");

  fclose(pagefile);
  fclose(textfile);
  if (!NOTOC)
      fclose(indexfile);
}


/*******************************************************************************
 void SetPageNum() : 
 *******************************************************************************/

void SetPageNum()
{
    int page_num = GetNumber();
/*    fprintf(f, "<A NAME=PN%d></A>", page_num); */
}


/*******************************************************************************
 void InsertFootnote() : 
 *******************************************************************************/

void InsertFootnote()
{
    char sss[200]; 

    if (!in_string)
	PrintHeaderTypeStart();
    in_string = TRUE;
	    
    footnote_use_nr++;
    if (in_table)
	{
	    sprintf(sss, "<A HREF=\"%s.html#FN%d%c\">(%c)</A>", 
		    TRANSFilename, current_table->id, 
		    footnote_use_nr + 'a' - 1, footnote_use_nr + 'a' - 1);
	}
    else
	{
	    sprintf(sss, "<A HREF=\"%s.html#FN%d\">(%d)</A>", TRANSFilename, footnote_use_nr, footnote_use_nr);
	}
    strcat(paragraph, sss); 
}


/*******************************************************************************
 void PrintFootnotes() : 
 *******************************************************************************/

void PrintFootnotes()
{
    int i;
    char *language = getenv("DOCUMENTLANGUAGE");

    if (language && language == "NO")	
	fprintf(pagefile, "\n<HR><h3>Fotnoter</h3>\n");
    else
	fprintf(pagefile, "\n<HR><h3>Footnotes</h3>\n");
    fprintf(pagefile, "<DL COMPACT>\n");
    for (i=1; i<=footnote_def_nr; i++)
	{	
	    fprintf(pagefile, "<DT><A NAME=FN%d>(%d)</A><DD>%s\n", i, i, footnote[i]);
	    free(footnote[i]);
	}
    fprintf(pagefile, "</DL>\n");
    footnote_def_nr = 0;
}


/*******************************************************************************
 void NewFootnoteStart() : 
 *******************************************************************************/

void NewFootnoteStart()
{
    footnote_def_nr ++;
    footnote[footnote_def_nr] = calloc(1000, 1);    
    footnote[footnote_def_nr][0] = 0;
    in_footnote = TRUE;
}

/*******************************************************************************
 void NewFootnoteEnd() : 
 *******************************************************************************/

void NewFootnoteEnd()
{
    in_footnote = FALSE;
}


/*******************************************************************************
 void InsertReferenceStart() : 
 *******************************************************************************/

void InsertReferenceStart()
{
    /* Does nothing */
}


/*******************************************************************************
 void SetReferenceNumber() : 
 *******************************************************************************/

void SetReferenceNumber()
{
    reference_number = GetNumber();
}

/*******************************************************************************
 void SetReferenceFile() : 
 *******************************************************************************/

void SetReferenceFile()
{
    char ref_string[300];
    GetReferenceFile();
    sprintf(ref_string, "<A HREF=\"%s#REF%d\">", reference_filename, reference_number);
    if (!in_string)
	PrintHeaderTypeStart();
    in_string = TRUE;
    strcat(paragraph, ref_string);
}

/*******************************************************************************
 void InsertReferenceEnd() : 
 *******************************************************************************/

void InsertReferenceEnd()
{
    char ref_string[300];
    sprintf(ref_string, "</A>");
    if (!in_string)
	PrintHeaderTypeStart();
    in_string = TRUE;
    strcat(paragraph, ref_string);
}


/*******************************************************************************
  void AFramesStart() : 
        Input  : 
	Output : 
 *******************************************************************************/

void AFramesStart()
{

}


/*******************************************************************************
  void AFramesEnd() :
        Input  : 
	Output : 
 *******************************************************************************/

void AFramesEnd()
{
    int i;
    struct stat stbuf;

    if (num_frames)
	{
	    sprintf(sys_call, "mif2gif ");		/* Translate to gif file */
	    for (i = 0; i < num_frames; i++)
		{
		    strcat(sys_call, framename[i]);
		    strcat(sys_call, " ");
		}
	    system(sys_call);
	    
	    /* Remove empty gifs (has length 35 or 0) */
	    
	    for (i = 0; i < num_frames; i++)
		{
		    strcpy(frame_filename, framename[i]);
		    frame_filename[strlen(frame_filename) - 3] = 'g';
		    if (stat(frame_filename, &stbuf) == 0)
			if (stbuf.st_size == 35 || stbuf.st_size == 0) 
			    {
				printf("trans: Deleting empty gif file: %s\n", frame_filename);
				unlink(frame_filename);
			    }
		}
	}
}


/*******************************************************************************
  void FrameStart() : 
        Input  : 
	Output : 
 *******************************************************************************/

void FrameStart()
{
    textrect_count = 0;
    frame_id = GetIDNextLine();
    in_frame = TRUE;

    if (NOGRAPHICS)
	return;

    sprintf(frame_filename, "%s.figure.id.%d.mif", TRANSFilename, frame_id);
    sprintf(sys_call, "cp %s.tmp.head.0011  %s", TRANSFilename, frame_filename);
    printf ("trans: Making FrameMaker MIF file \"%s\"\n", frame_filename);
    system(sys_call);

    framef=fopen(frame_filename, "a");
    if (!framef)
	{
	    fprintf(stderr, "trans: Could not open file \"%s\"\n", frame_filename);
	    exit(-1);
	}

    fprintf(framef, "%s\n", yytext);
}


/*******************************************************************************
  void FrameEnd() :
        Input  : 
	Output :
 *******************************************************************************/

void FrameEnd()
{
    int i;

/*    fclose(f); */

    in_frame = FALSE;

    if (NOGRAPHICS)
	return;

    fprintf(framef, "%s\n", yytext);
    fprintf(framef, "<TextFlow \n");
    fprintf(framef, " <Para \n");
    fprintf(framef, "  <PgfTag `frame2htmlspecialparagraphformat'>\n");
    fprintf(framef, "  <ParaLine \n");
    fprintf(framef, "   <AFrame %d>\n", frame_id);
    fprintf(framef, "  >\n > # end of Para\n");
    fprintf(framef, "> # end of TextFlow\n");
    fclose(framef);

    fprintf(stderr, "trans: Number of textrects: %d\n", textrect_count);

    if (textrect_count > 0)
	{
	    sprintf(sys_call, "addtextrect %s %s", TRANSFilename, frame_filename);

	    for (i=0; i<textrect_count; i++) 
		{
		    char temp_num[10];
		    strcat(sys_call, " ");
		    sprintf(temp_num, " %d%c", textrect_id[i], 0);
		    strcat(sys_call, temp_num);
		}
	    system(sys_call);
	}


    framef=fopen(frame_filename, "a");
    if (!framef)
	{
	    fprintf(stderr, "Could not open file \"%s\"\n", frame_filename);
	    exit(-1);
	}
    fprintf(framef, "# End of MIFFile\n");
    fclose(framef);
    framename[num_frames++] = (char *) strdup(frame_filename);
}


/*******************************************************************************
  void AFrameUse() :
        Input  : 
	Output : 
 *******************************************************************************/

void AFrameUse()
{
    int framenr, fframety;
    framenr = GetID();
    fframety = GetFrameType(framenr);
    if (fframety == INLINEFRAME || in_table)
	{
	    InsertInlineFigure(f, framenr);
	}
    else
	{
	    tabfignum[tabfig_num] = GetID();
	    tabfig[tabfig_num++] = FIGURE;
	}
}


/*******************************************************************************
  void TablesStart() : All tables are skipped
        Input  : 
	Output : Indicate parsing tables.
 *******************************************************************************/

void TablesStart()
{
    in_table = TRUE;
}


/*******************************************************************************
  void TablesEnd() :
        Input  : 
	Output : No more tables.
 *******************************************************************************/

void TablesEnd()
{
    in_table = FALSE;
}


/*******************************************************************************
  void TableUse() :
        Input  : 
	Output : Prints information concerning missing table.
 *******************************************************************************/

void TableUse()
{
    tabfignum[tabfig_num] = GetID();
    tabfig[tabfig_num++] = TABLE;
}


/*******************************************************************************
  void SetMarker() :
        Input  : Uses yytext
	Output : marker_number = ID of marker
	OR for hypertext: marker_text.
 *******************************************************************************/

void SetMarker()
{
    if (marker_type == 8)       /* HyperText Link */
        {
            char *content;
            content = GetContents();
            if (content)
                {
                    char *hypcommand;          /* Used to get tokens from string */
		    char *hypfilename;
		    char *hyplink;
                    char *c;            /* Pointer to ':' in linkspec */

                    /*
                     * Parse the tokens out of the string looking for
                     * openlink or gotolink
                     */

                    hypcommand = strtok(content, " \t");
                    if ((!strcasecmp(hypcommand, "openlink")) ||			/* Handle openlink and gotolink */
                        (!strcasecmp(hypcommand, "gotolink"))) 
			{
			    hypfilename = (char *)strtok(NULL, ":");
			    if (!hypfilename)
				{
				    fprintf(stderr, "trans: Hypertext \"%s\" with no parameters or space. Removed.\n", hypcommand);
				}
			    else
				{
				    hyplink = strtok(NULL, ":");
				    if (hyplink)					/* Openlink or gotolink with file and linkspec */
					{
					    marker_text = calloc(1, 1000);
					    if (c = (strstr(hypfilename, ".doc"))) 		/* Remove .doc */
						{
						    *c = 0;
						}
					    if (!strcasecmp(hyplink, "firstpage"))
						{
						    sprintf(marker_text, "<A HREF=\"%s.html\">", hypfilename);
						}
					    else if (!strcasecmp(hyplink, "lastpage"))
						{
						    sprintf(marker_text, "<A HREF=\"%s.html#ENDFILE\">", hypfilename);
						}
					    else
						{
						    sprintf(marker_text, "<A HREF=\"%s.html#%s\">", hypfilename, hyplink);
						}
					}
				    else 						/* Openlink or gotolink with only a linkspec */
					{
					    marker_text = calloc(1, 1000);
					    hyplink = hypfilename;
					    if (!strcasecmp(hyplink, "firstpage"))
						{
						    sprintf(marker_text, "<A HREF=\"%s.html\">", TRANSFilename);
						}
					    else if (!strcasecmp(hyplink, "lastpage"))
						{
						    sprintf(marker_text, "<A HREF=\"%s.html#ENDFILE\">", TRANSFilename);
						}
					    else
						{
						    sprintf(marker_text, "<A HREF=\"%s.html#%s\">", TRANSFilename, hyplink);
						}
					}
				}
			}
		    else if (!strcasecmp(hypcommand, "newlink"))
			{
			    hyplink = (char *)strtok(NULL, ":");
			    if (hyplink)
				{		
				    marker_text = calloc(1, 1000);
				    sprintf(marker_text, "<A NAME=\"%s\">", hyplink);
				}
			    else
				fprintf(stderr, "trans: FrameMaker hypertext command \"newlink\" with no parameters. Ignored.");
			}
                    else if ((!strcasecmp(hypcommand, "alert")) ||
			     (!strcasecmp(hypcommand, "matrix")) ||
			     (!strcasecmp(hypcommand, "nextpage")) ||
			     (!strcasecmp(hypcommand, "popup")) ||
			     (!strcasecmp(hypcommand, "previouslink")) ||
			     (!strcasecmp(hypcommand, "previouspage")) ||
			     (!strcasecmp(hypcommand, "quit")) ||
			     (!strcasecmp(hypcommand, "quitall")) ||
			     (!strcasecmp(hypcommand, "message")))
			{
			    fprintf(stderr, "trans: The FrameMaker hypertext command \"%s\" is not converted.\n", hypcommand);
			}
		    else
			{
			    /* 
			     * Not a openlink or a gotolink, use the whole thing
			     * as a URL.
			     */

			    marker_text = calloc(1, 1000);
			    sprintf(marker_text, "<A HREF=\"%s\">", content);
			}
                    free(content);
                }
        }
    else if (marker_type == 9)
        marker_number = GetNumber();
    else
	{
	    fprintf(stderr, "Ignored marker of type %d.\n", marker_type); 
	}
}



/*******************************************************************************
  void SetMarkerType() :
        Input  : Uses yytext
	Output : marker_type = type of marker
 *******************************************************************************/

void SetMarkerType()
{
    marker_type = atoi(yytext + 6);
}



/*******************************************************************************
  char *ChangeToSymbol (str) :
 *******************************************************************************/

char *ChangeToSymbol( str )
      char *str;
{
    int i = 0;
    int j = 0;
    long val;
    char *new_str = calloc(strlen(str) * 5 + 1, sizeof(char) );
    while (str[i]) 
	{
	    val = str[i];
	    if (str[i]=='\\' && str[i+1] == 'x' && str[i+4] == ' ')			      
		{
		    new_str[j++] = str[i++];
		    new_str[j++] = str[i++];
		    new_str[j++] = str[i++];
		    new_str[j++] = str[i++];
		    new_str[j++] = '\\';
		    i++;
		}
	    else
		{
		    char *tempc = calloc(10,1);
		    sprintf(tempc, "\\x%x\\", val);
		    strcat(new_str, tempc);
		    free(tempc);
		    j+=5;
		    i++;
		}
	}
    /* free(str); */
    return(new_str);
}


/*******************************************************************************
  char *GetContents() :
        Input  : Uses yytext
	Output : Text in yytext between ` '
 *******************************************************************************/

char *GetContents()
{
    int i = 0, j = 0;
    char *quoted;

    quoted = malloc(strlen(yytext) * 5 );		/* To be able to handle < > and & */

    while(yytext[i] && yytext[i] != 96)
	{
	    i++;
	}

    if (yytext[i] == 96)
	{
	    i++;
	    while(yytext[i] && yytext[i] != 39)
		{
		    if (yytext[i] == '<') 
			{
			    quoted[j++] = '&';
			    quoted[j++] = 'l';
			    quoted[j++] = 't';
			    quoted[j++] = ';';
			}
		    else if (yytext[i] == '\\' && yytext[i+1] == '>')
			{
			    quoted[j++] = '&';
			    quoted[j++] = 'g';
			    quoted[j++] = 't';
			    quoted[j++] = ';';		
			    i++;
			}
		    else if (yytext[i] == '&')
			{
			    quoted[j++] = '&';
			    quoted[j++] = 'a';
			    quoted[j++] = 'm';
			    quoted[j++] = 'p';
			    quoted[j++] = ';';
			}
                   else if (yytext[i] == '\\' && yytext[i+1] == 'q')
                       {
                           /* ejr:  handle smart quote \q */
                           quoted[j++] = '\'';
                           i++;
                       }
                   else if (yytext[i] == '\\' && yytext[i+1] == '\\')
                       {
                           /* ejr: handle escaped slash \\ */
                           quoted[j++] = '\\';
                           i++;
                       }
		    else		    
			quoted[j++] = yytext[i];
		    i++;
		}	
	    if (yytext[i] == 39)
		{
		    quoted[j] = 0;
		    if (font_type == SYMBOL)
			return ChangeToSymbol(quoted);
		    else
			return quoted;
		}
	    else
		{
		    free(quoted);
		    return NULL;
		}
	}			
    else 
	{
	    free(quoted);
	    return NULL;
	}
}	


/*******************************************************************************
  char *GetHdr() :
        Input  : Uses yytext
	Output : Text in yytext between ` '
 *******************************************************************************/

char *GetHdr()
{
    int i = 0, j = 0;
    char *quoted;

    quoted = malloc(strlen(yytext) * 5 );		/* To be able to handle < > and & */

    while(yytext[i] && yytext[i] != 96)
	{
	    i++;
	}

    if (yytext[i] == 96)
	{
	    i++;
	    while(yytext[i] && yytext[i] != 39)
		{
		    if (yytext[i] == '<') 
			{
			    quoted[j++] = '&';
			    quoted[j++] = 'l';
			    quoted[j++] = 't';
			    quoted[j++] = ';';
			}
		    else if (yytext[i] == '\\' && yytext[i+1] == '>')
			{
			    quoted[j++] = '&';
			    quoted[j++] = 'g';
			    quoted[j++] = 't';
			    quoted[j++] = ';';		
			    i++;
			}
		    else if (yytext[i] == '&')
			{
			    quoted[j++] = '&';
			    quoted[j++] = 'a';
			    quoted[j++] = 'm';
			    quoted[j++] = 'p';
			    quoted[j++] = ';';
			}
		    else		    
			quoted[j++] = yytext[i];
		    i++;
		}	
	    if (yytext[i] == 39)
		{
		    quoted[j] = 0;
		    return quoted;
		}
	    else
		{
		    free(quoted);
		    return NULL;
		}
	}			
    else 
	{
	    free(quoted);
	    return NULL;
	}
}	


/*******************************************************************************
  int GetNumber() :
        Input  : Uses yytext
	Output : Number identifying marker
 *******************************************************************************/

int GetNumber()
{
    int i = 0;

    while(yytext[i] && yytext[i] != 96)
	{
	    i++;
	}

    if (yytext[i] == 96)
	{
	    i++;
	    return atoi(&(yytext[i]));
	}			
    else return 0;
}


/*******************************************************************************
  ParagraphNumber() :
        Input  : Uses yytext
	Output : Paragraph number string with \t removed
 *******************************************************************************/

void ParagraphNumber()
{
    int i = 0, j = 0;

    while(yytext[i] && yytext[i] != 96)
	{
	    i++;
	}

    if (yytext[i] == 96)
	{
	    i++;
	    while (yytext[i] && yytext[i] != 39)
		{
		    if (yytext[i] == 92)			
			{
			    if (yytext[i+1] == 't')
				{	
				    para_num_string[j++] = ' ';
				    i++;
				}
			    else
				para_num_string[j++] = yytext[i];
			}
		    else
			para_num_string[j++] = yytext[i];
		    i++;
		}
	    para_num_string[j++] = ' ';
	    para_num_string[j] = 0;
	    if (!strcmp(para_num_string, "- "))
		strcpy(para_num_string, "<LI>");
	}			
    else para_num_string[0] = 0;
}


/*******************************************************************************
  void GetReferenceFile() :
        Input  : Uses yytext
	Output : Reference file
 *******************************************************************************/

void GetReferenceFile()
{
    int i = 0, j = 0;
    char *ptr;

    while(yytext[i] && yytext[i] != 96)
	{
	    i++;
	}

    if (yytext[i] == 96)
	{
	    i++;
	    while (yytext[i] && yytext[i] != 39)
		{
		    if (yytext[i] == '<')			
			{
			    if (yytext[i+1] == 'c')
				{	
				    i++;		/* Get by <c\> */
				    i++;
				    i++;
				    i++;
				    while (yytext[i] && yytext[i] != '<' && yytext[i] != 39)
					{
					    reference_filename[j++] = yytext[i++];
					}
				    reference_filename[j]=0;
				    if ((ptr = (char *) strstr(reference_filename, ".doc")) != NULL)
					{
					    *ptr = 0;
					}
				    else if ((ptr = (char *) strstr(reference_filename, ".mif")) != NULL)
					{
					    *ptr = 0;
					}
				    /* if (!strcmp(TRANSFilename, reference_filename))
				           reference_filename[0] = 0;
				       else	*/
				    strcat(reference_filename, ".html");
				    return;
				}
			}
		    i++;
		}
	    /* reference_filename[0] = 0;   */    /* Jump inside file */
	    strcpy(reference_filename, TRANSFilename);
	    strcat(reference_filename, ".html");
	}			
    else 
	{
	    /* reference_filename[0] = 0;  */    /* Jump inside file */
	    strcpy(reference_filename, TRANSFilename);
	    strcat(reference_filename, ".html");
	}
}

	    

/*******************************************************************************
  NewParagraph() :
 *******************************************************************************/

void NewParagraph()
{
}


/*******************************************************************************
  EndParagraph() : If the paragraph contained text, then output the end paragraph
                   tag. 
 *******************************************************************************/

void EndParagraph()
{
    int i;
    
    ResetFont();

    if (in_string || para_num_string[0])
	{
	    in_string = FALSE;
	    PrintHeaderTypeEnd();
	}

    for (i = 0; i < tabfig_num; i++)
	{
	    if (tabfig[i] == FIGURE)
		{
		    InsertFigure(f, tabfignum[i]);
		}
	    else
		{
		    InsertTable(f, tabfignum[i]);
		}
	}
	
    font_type = NORMAL;
    tabfig_num = 0;
	    
    para_num_string[0] = 0;		/* Ensure that old paragraph numbering is not used */
}


/*******************************************************************************
  ChangeOutputFile() :
 *******************************************************************************/

void ChangeOutputFile()
{
    f = textfile;
}


/*******************************************************************************
  int GetIDNextLine() :
        Input  : Uses yytext
	Output : 
 *******************************************************************************/

int GetIDNextLine()
{
    int i = 0;

    while(yytext[i] && yytext[i] != 'D')
	{
	    i++;
	}
    i++;
    return atoi(&(yytext[i]));
}


/*******************************************************************************
  PrintTextRectID() :
        Input  : Uses yytext
	Output : "<<ID>>, where ID is the ID of the textrect
 *******************************************************************************/

void PrintTextRectID()
{
    int id = GetIDNextLine();
    if (header == FIGURE && frame_id_arr[id])
	fprintf(f, "\n>>%d<<%d>>", frame_id_arr[id], id);
    else 
	fprintf(f, "\n<<%d>>", id);
    textrect = id;
}


/*******************************************************************************
  void FontStart() : Reset Font settings.
 *******************************************************************************/

void FontStart()
{
    ResetFont();
}


/*******************************************************************************
  void ResetFont() : Reset Font settings.
 *******************************************************************************/

void ResetFont()
{
    if (font_bold)
	{
	    strcat(paragraph, "</B>");
	    font_bold = FALSE;
	}
    if (font_italic)
	{
	    strcat(paragraph, "</I>");
	    font_italic = FALSE;
	}
    if (font_underline)
	{
	    strcat(paragraph, "</U>");
	    font_underline = FALSE;
	}
    if (font_supscript)
	{
	    strcat(paragraph, "</em>");
	    font_supscript = FALSE;
	}
    if (font_subscript)
	{
	    strcat(paragraph, "</em>");
	    font_subscript = FALSE;
	}
}


/*******************************************************************************
  void FontEnd() :
 *******************************************************************************/

void FontEnd()
{
}


/*******************************************************************************
  ChangeFont() :
 *******************************************************************************/

void ChangeFont()
{
    char font_name[50];
    int i = 10;			/* Skip "<FFamily `" */
    int j = 0;

    while (yytext[i] && yytext[i] != 39)
	{
	    font_name[j++] = yytext[i];
	    i++;
	}
    font_name[j] = 0;
    if (yytext[i])
	{
	    if (!strcmp(font_name, "Symbol"))
		{
		    font_type = SYMBOL;
		    /* fprintf(stderr, "Change to SYMBOL\n" ); */
		}

	    else if (!strcmp(font_name, "ZapfDingbats"))
		{
		    font_type = DINGBATS;
		    /* fprintf(stderr, "Change to DINGBATS\n" ); */
		}
	    else 
		{
		    font_type = NORMAL;
		    /* fprintf(stderr, "Change to NORMAL\n" ); */
		}
	}
}


/*******************************************************************************
  InsertSpecialChar() :
 *******************************************************************************/

void InsertSpecialChar()
{
    char char_name[50];
    int i = 6;			/* Skip "<Char " */
    int j = 0;

    if (!in_string)
	PrintHeaderTypeStart();
    in_string = TRUE;
	    
    while (yytext[i] && yytext[i] != 32)
	{
	    char_name[j++] = yytext[i];
	    i++;
	}
    char_name[j] = 0;
    if (font_type == SYMBOL)
	{
	    if (yytext[i])
		{
		    if (!strcmp(char_name, "EmDash"))
			strcat(paragraph, kEM_DASH_PREF);
		    else if (!strcmp(char_name, "EnDash"))
			strcat(paragraph, kEN_DASH_PREF);
		    else if (!strcmp(char_name, "EmSpace"))
			strcat(paragraph, "   ");
		    else if (!strcmp(char_name, "EnSpace"))
			strcat(paragraph, "  ");
		    else if (!strcmp(char_name, "HardSpace"))
			strcat(paragraph, " ");
		    else if (!strcmp(char_name, "Pound"))
			strcat(paragraph, "\\xa3\\");
		    else if (!strcmp(char_name, "Cent"))
			strcat(paragraph, "\\xa2\\");
		    else if (!strcmp(char_name, "Yen"))
			strcat(paragraph, "\\xb4\\");
		    else if (!strcmp(char_name, "HardReturn"))
                       {
			   /* ejr:  long lists sometimes are broken with
			    *       a hard return instead of a tab.
			    */
                           if (header >= GLOSSARY && header <= COMGLOSSARY3 || header == ACRONYM)
                               strcat(paragraph, "<DD>");
                           else
                               strcat(paragraph, "<BR>\n");
                       }
		    else if (!strcmp(char_name, "Tab"))
			{
			    if (header >= GLOSSARY && header <= COMGLOSSARY3 || header == ACRONYM)
				strcat(paragraph, "<DD>");
			    else
				strcat(paragraph, "  ");
			}
		    else if (in_table && (!strcmp(char_name, "SoftHyphen") || !strcmp(char_name, "HardHyphen")))
			strcat(paragraph, "-");
		    else if (!(strcmp(char_name, "SoftHyphen") && 
			     strcmp(char_name, "HardHyphen") &&
			     strcmp(char_name, "DiscHyphen") && 
			     strcmp(char_name, "NoHyphen")))
			{
			    /* fprintf(stderr, "trans: Special SYMBOL character not handled: \"%s\"\n", char_name); */
			}
		}
	}
    else if (font_type == DINGBATS)
	{
	    fprintf(stderr, "\ntrans: Special Dingbat char not handled: \"%s\"\n", char_name);
	}
    else
	{
	    if (yytext[i])
		{
		    if (!strcmp(char_name, "EmDash"))
			strcat(paragraph,  kEM_DASH_PREF);
		    else if (!strcmp(char_name, "EnDash"))
			strcat(paragraph, kEN_DASH_PREF);
		    else if (!strcmp(char_name, "EmSpace"))
			strcat(paragraph, "   ");
		    else if (!strcmp(char_name, "EnSpace"))
			strcat(paragraph, "  ");
		    else if (!strcmp(char_name, "HardSpace"))
			strcat(paragraph, " ");
		    else if (!strcmp(char_name, "Pound"))
			strcat(paragraph, "\xa3");
		    else if (!strcmp(char_name, "Cent"))
			strcat(paragraph, "\xa2");
		    else if (!strcmp(char_name, "Yen"))
			strcat(paragraph, "\xa5");
		    else if (!strcmp(char_name, "HardReturn"))
			{
			    /* ejr:  long lists sometimes are broken with
			     *       a hard return instead of a tab.
			     */
			    if (header >= GLOSSARY && header <= COMGLOSSARY3 ||header == ACRONYM)
                               strcat(paragraph, "<DD>");
                           else
                               strcat(paragraph, "<BR>\n");
			}
		    else if (!strcmp(char_name, "Tab"))
			{
			    if (header >= GLOSSARY && header <= COMGLOSSARY3 || header == ACRONYM)
				strcat(paragraph, "<DD>");
			    else
				strcat(paragraph, "  ");
			}
		    else if (!(strcmp(char_name, "SoftHyphen") && strcmp(char_name, "HardHyphen") &&
			     strcmp(char_name, "DiscHyphen") && strcmp(char_name, "NoHyphen")))
			{
			    /* fprintf(stderr, "trans: Special character not handled: \"%s\"\n", char_name); */
			}
		}
	}
}


/*******************************************************************************
  DoFrame() :
 *******************************************************************************/

void DoFrame()
{

    if (NOGRAPHICS)
	return;

    if (in_frame)
	{
	    fprintf(framef, "%s\n", yytext);
	}
}


/*******************************************************************************
  TextRect() :
 *******************************************************************************/

void TextRect()
{
    if (in_frame)
	{
	    textrect_id[textrect_count++] = GetIDNextLine();
	    frame_id_arr[textrect_id[textrect_count-1]] = frame_id;
	}
}


/*******************************************************************************
  InsertFigure() :
 *******************************************************************************/

void InsertFigure(f, id)
      FILE *f;
      int id;
{
    sprintf(frame_filename, "%s.figure.id.%d.gif", TRANSFilename, id);
    fprintf(f, "<A HREF=\"%s\"><img src=\"%s\"></A><p>\n", 
	    frame_filename, frame_filename);
}


/*******************************************************************************
  InsertInlineFigure() :
 *******************************************************************************/

void InsertInlineFigure(f, id)
      FILE *f;
      int id;
{
    char frame_link[1000];
    if (!in_string)
	PrintHeaderTypeStart();
    in_string = TRUE;
	    
    if (in_table)
	{
	    sprintf(frame_link, "<A HREF = \"%s.figure.id.%d.gif\">(fig)</A>", 
		    TRANSFilename, id);	    
	}
    else
	{
	    sprintf(frame_link, "<A HREF = \"%s.figure.id.%d.gif\"><img src=\"%s.figure.id.%d.gif\"></A>", 
		    TRANSFilename, id, TRANSFilename, id);
	}
    strcat(paragraph, frame_link);
}


/*******************************************************************************
  SetFrameType() :
 *******************************************************************************/

void SetFrameType()
{
    if (NOGRAPHICS)
        return;

    frametype[num_frames][0] = frame_id;
    if (strstr(yytext, "Inline"))
	{
	    frametype[num_frames][1] = INLINEFRAME;
	}
    else
	{
	    frametype[num_frames][1] = OTHERFRAME;
	}
    fprintf(framef, "%s\n", yytext);
}


/*******************************************************************************
  GetFrameType() :
 *******************************************************************************/

int GetFrameType(framenr)
      int framenr;
{
    int i;
    for (i=0; i<num_frames; i++)
	{
	    if (frametype[i][0] == framenr)
		return frametype[i][1];
	}
    return -1;
}


/*******************************************************************************
  ChangeFontWeight() :
 *******************************************************************************/

void ChangeFontWeight()
{
    int new_font_bold;

    if (strstr(yytext, "Bold"))
	new_font_bold = TRUE;
    else
	new_font_bold = FALSE;

    if (font_bold != new_font_bold)
	{
	    if (font_bold)
		strcat(paragraph, "</B>");
	    else
		strcat(paragraph, "<B>");
	}
    font_bold = new_font_bold;
}


/*******************************************************************************
  ChangeFontAngle() :
 *******************************************************************************/

void ChangeFontAngle()
{
    int new_font_italic;

    if (strstr(yytext, "Italic") || strstr(yytext, "Oblique")) /* AFEB added Oblique */
	new_font_italic = TRUE;
    else
	new_font_italic = FALSE;

    if (font_italic != new_font_italic)
	{
	    if (font_italic)
		strcat(paragraph, "</I>");
	    else
		strcat(paragraph, "<I>");
	}
    font_italic = new_font_italic;
}


/*******************************************************************************
  ChangeFontUnderline() :
 *******************************************************************************/

void ChangeFontUnderline()
{
    int new_font_underline;

    if (strstr(yytext, "Yes"))
	new_font_underline = TRUE;
    else
	new_font_underline = FALSE;

    if (font_underline != new_font_underline)
	{
	    if (font_underline)
		strcat(paragraph, "</U>");
	    else
		strcat(paragraph, "<U>");
	}
    font_underline = new_font_underline;
}


/*******************************************************************************
  ChangeFontSupscript() :
 *******************************************************************************/

void ChangeFontSupscript()
{
    int new_font_supscript;

    if (strstr(yytext, "Yes"))
	new_font_supscript = TRUE;
    else
	new_font_supscript = FALSE;

    if (font_supscript != new_font_supscript)
	{
	    if (font_supscript)
		strcat(paragraph, "</em>");
	    else
		strcat(paragraph, "<em sup>");
	}
    font_supscript = new_font_supscript;
}


/*******************************************************************************
  ChangeFontSupscript() :
 *******************************************************************************/

void ChangeFontSubscript()
{
    int new_font_subscript;

    if (strstr(yytext, "Yes"))
	new_font_subscript = TRUE;
    else
	new_font_subscript = FALSE;

    if (font_subscript != new_font_subscript)
	{
	    if (font_subscript)
		strcat(paragraph, "</em>");
	    else
		strcat(paragraph, "<em sub>");
	}
    font_subscript = new_font_subscript;
}



/*******************************************************************************
 void EndTextFlow() : 
 *******************************************************************************/

void EndTextFlow()
{
    
    if (depth) Ascend(depth);
    if (in_reference)
	{
	    fprintf(f, "</DL>\n");
	    in_reference = FALSE;
	}
    else if (in_glossary)
	{
	    switch ( in_glossary)
		{
		  case GLOSSARY:
		  case COMGLOSSARY:
		    fprintf(f, "</DL>\n");
		    break;
		  case GLOSSARY2:
		  case COMGLOSSARY2:
		    fprintf(f, "</DL></UL>\n");
		    break;
		  case GLOSSARY3:
		  case COMGLOSSARY3:
		    fprintf(f, "</DL></UL></UL>\n");
		    break;
		    
		  default:
		    fprintf(stderr, "Don't know glossary state %d\n", in_glossary);
		}
	    in_glossary = FALSE;
	    
	}
    else if (in_program)
	{
	    fprintf(f, "</PRE>\n");
	    in_program = FALSE;
	}
    else if (in_acronym)
	{
	    fprintf(f, "</DL>\n");
	    in_acronym = FALSE;
	}
}



/*******************************************************************************
  void InlineFrameStart() : 
        Input  : 
	Output : 
 *******************************************************************************/

void InlineFrameStart()
{
    textrect_count = 0;
    frame_id = inlineframe_num++;
    in_frame = TRUE;

    if (NOGRAPHICS)
        return;

    sprintf(frame_filename, "%s.figure.id.I%d.mif", TRANSFilename, frame_id);
    sprintf(sys_call, "cp %s.tmp.head.0011  %s", TRANSFilename, frame_filename);
    printf ("trans: Making FrameMaker MIF file \"%s\"\n", frame_filename);
    system(sys_call);

    framef=fopen(frame_filename, "a");
    if (!framef)
	{
	    fprintf(stderr, "trans: Could not open file \"%s\"\n", frame_filename);
	    exit(-1);
	}

    fprintf(framef, "%s\n", yytext);
}


/*******************************************************************************
  void InlineFrameEnd() :
        Input  : 
	Output :
 *******************************************************************************/

void InlineFrameEnd()
{
    int i;
    struct stat stbuf;

    in_frame = FALSE;

    fprintf(framef, "%s\n", yytext);
    fclose(framef);

    fprintf(stderr, "trans: Number of textrects: %d\n", textrect_count);

    if (textrect_count > 0)
	{
	    sprintf(sys_call, "addtextrect %s %s", TRANSFilename, frame_filename);

	    for (i=0; i<textrect_count; i++) 
		{
		    char temp_num[10];
		    strcat(sys_call, " ");
		    sprintf(temp_num, " %d%c", textrect_id[i], 0);
		    strcat(sys_call, temp_num);
		}
	    system(sys_call);
	}


    framef=fopen(frame_filename, "a");
    if (!framef)
	{
	    fprintf(stderr, "Could not open file \"%s\"\n", frame_filename);
	    exit(-1);
	}
    fprintf(framef, "# End of MIFFile\n");
    fclose(framef);
    
    sprintf(sys_call, "mif2gif ");		/* Translate to gif file */
    strcat(sys_call, frame_filename);
    strcat(sys_call, " ");
    system(sys_call);
	    
    /* Remove empty gifs (has length 35 or 0) */
	    
    frame_filename[strlen(frame_filename) - 3] = 'g';

    if (stat(frame_filename, &stbuf) == 0)
	if (stbuf.st_size == 35 || stbuf.st_size == 0) 
	    {
		printf("trans: Deleting empty gif file: %s\n", frame_filename);
		unlink(frame_filename);
	    }



    fprintf(f, "\n<A HREF = \"%s\"><img src=\"%s\"></A>\n", 
	    frame_filename, frame_filename);
}


/*******************************************************************************
  void StripSpace() :
 *******************************************************************************/

void StripSpace(txt)
      char *txt;
{
    int length = strlen(txt);
    int i;
    char *txtp;
    
    txtp = txt + length - 1;

    for (i=length-1; i>=0; i--, txtp--)
	{
	    if (*txtp == 32)
		*txtp = 0;
	    else
		return;
	}
}


/*******************************************************************************
  void GetTextOnly :
 *******************************************************************************/

char *GetTextOnly(from)
      char *from;
{
    char *fromp = from;
    char *to = calloc(strlen(from) + 1, 1);
    char *top = to;

    while (*fromp)
	{
	    if (*fromp == '<')
		{
		    while (*fromp && *fromp != '>') fromp++;
		    if (!*fromp) return to;
		}
	    else
		{
		    *top = *fromp;
		    top++;
		}
	    fromp++;
	}
    return to;
}
	

/*******************************************************************************
  void HandleParaLine()
 *******************************************************************************/

void HandleParaLine()
{
    if (in_table && !in_footnote) 
	NewCellLine(); 
    if (header == PROGRAM && in_string) 
	strcat(paragraph, "\n");
}


/*************************************************************************
 void main: added by popular demand
 *************************************************************************/

void main()
{

yyparse();

}


/***************************************************************************
 Variable handling
 ***************************************************************************/


/*\ Some simple variable handling
  \*/

#define MAX_VARIABLES 512

typedef struct sVarLookup
{
    char      *fVarName,
    *fVarDef;
}
TVarLookup;

TVarLookup varLookup[ MAX_VARIABLES] = { NULL,};

int numVariables = 0;


char *lastVariable = NULL;



/*******************************************************************************
  char *GetVarDef() :
  Input  : Uses yytext
  Output : Text in yytext between ` '
  *******************************************************************************/

char *GetVarDef()
{
    int i = 0, j = 0;
    char *quoted;
    
    quoted = malloc(strlen(yytext) * 5 );             /* To be able to handle < > and & */
    
    while(yytext[i] && yytext[i] != 96)
	{
	    i++;
	}
    
    if (yytext[i] == 96)
	{
	    i++;
	    while(yytext[i] && yytext[i] != 39)
		{
		    if (yytext[i] == '<')
			{
			    while( yytext[i] && !(yytext[i] == '\\' && yytext[i+1] == '>') )
				i++;
			    if ( yytext[i] && yytext[i] == '\\')
				i++;
			}
		    else if (yytext[i] == '\\' && yytext[i+1] == 'q')
			{
			    quoted[j++] = '\'';
			    i++;
			}
		    else if (yytext[i] == '\\' && yytext[i+1] == 'x')
			{
			    if ( yytext[i+2] == 'a' && yytext[i+3] == 'a')
				{
				    quoted[j++] = '(';
				    quoted[j++] = 't';
				    quoted[j++] = 'm';
				    quoted[j++] = ')';
				}
			    i++;
			    i++;
			    i++;
			}
		    else if (yytext[i] == '\\' && yytext[i+1] == '\\')
			{
			    quoted[j++] = '\\';
			    i++;
			}
		    else if (yytext[i] == '&')
			{
			    quoted[j++] = '&';
			    quoted[j++] = 'a';
			    quoted[j++] = 'm';
			    quoted[j++] = 'p';
			    quoted[j++] = ';';
			}
		    else
			quoted[j++] = yytext[i];
		    i++;
		}
	    if (yytext[i] == 39)
		{
		    quoted[j] = 0;
		    return quoted;
		}
	    else
		{
		    free(quoted);
		    return NULL;
		}
	}
    else
	{
	    free(quoted);
	    return NULL;
	}
}

/* Save a variable name */

SaveVariable()
{
    char      *theVariable;
    
    theVariable = GetContents();
    
    lastVariable = theVariable;
}

/* Save the variable's definition (follows name) */

SaveVariableDef()
{
    char      *theDef;
    
    theDef = GetVarDef();
    
    if ( numVariables < MAX_VARIABLES)
	{
	    varLookup[ numVariables].fVarName = lastVariable;
	    varLookup[ numVariables].fVarDef  = theDef;
	    numVariables++;
	}
}

/* Replaces variable name with its definition */

NoteVariName()
{
    char      *theVariable;
    int               loop;
    
    theVariable = GetContents();
    
    for ( loop = 0; loop < numVariables; loop++)
	if ( strcmp( theVariable, varLookup[ loop].fVarName) == 0)
	    strcat( paragraph, varLookup[ loop].fVarDef);
}



