%{
/*********************************************************/
/* FileName						 */
/* Copyright : Norwegian Telecom Research, 1993		 */
/* Programming : Jon Stephenson von Tetzchner		 */
/* This file can be changed and modified as long as this */
/* header is kept intact. This code can not be included	 */
/* in a commercial product.				 */
/*********************************************************/

#include <ctype.h>
#include <stdio.h>
#include "trans.h"

/*
  * Avoiding stack overflows...
  * Jan van der Steen (jansteen@cwi.nl)
  */

#define YYMAXDEPTH 4000
 

%}

%token MIFSTART
%token MIFEND
%token PAGE
%token PARASTART
%token PARALINE
%token PGFTAG
%token PGFNUMSTRING
%token STRING
%token PARAEND
%token FNOTEUSE
%token FNOTESTART
%token FNOTEEND
%token XREFSTART
%token XREFSRCTEXT
%token XREFSRCFILE
%token XREFEND
%token AFRAMESSTART
%token AFRAMESEND
%token FRAMESTART
%token INTFRAMESTART
%token FRAMEEND
%token FRAMETYPE
%token AFRAMEUSE
%token TABLESSTART
%token TABLESTART
%token TABLEID
%token TABLEFORMATSTART
%token TABLEFORMATEND
%token TABLENUMCOLUMNS
%token TABLETITLESTART
%token TABLETITLEEND
%token TABLEHEADSTART
%token TABLEHEADEND
%token TABLEFOOTSTART
%token TABLEFOOTEND
%token TABLEBODYSTART
%token TABLEROWSTART






















%token TABLECELLSTART
%token TABLECELLEND
%token TABLEROWEND
%token TABLEBODYEND
%token TABLEEND
%token TABLESEND
%token TABLEUSE
%token MARKERTYPE
%token MARKERTEXT
%token TEXTFLOWSTART
%token TEXTFLOWRECTID
%token TEXTFLOWEND
%token TEXTRECTIDUSE
%token MIFSTART
%token SPECIALCHAR
%token FFAMILY
%token FONTSTART
%token FONTEND
%token PGFFONTSTART
%token PGFFONTEND

%token FWEIGHT
%token FANGLE
%token FUNDERLINE
%token FSUPSCRIPT
%token FSUBSCRIPT

%token  VARISTART
%token  VARINAME
%token  VARIEND
%token  VARIFORMSSTART
%token  VARIFORMSTART
%token  VARIDEF
%token  VARIFORMEND
%token  VARIFORMSEND


%%

miffile		:	MIFSTART {StartPrint();} head pages textflow MIFEND {EndPrint();}
                ;	

head		:	skip head
                | 	TABLEFORMATSTART skips TABLEFORMATEND head
                |	aframes head
                |	tables head
                |       variformats head
                |	{/*  End Head */}	
                ;

aframes		:	AFRAMESSTART {AFramesStart();} frames AFRAMESEND {AFramesEnd();} 
                ;

frames		:	frame frames
                |
                ;

frame	        :	FRAMESTART {FrameStart();} framecontents FRAMEEND {FrameEnd();}
                ;

framecontents 	:	frameprint {DoFrame();} framecontents
                |	FRAMETYPE {SetFrameType();} framecontents
                |	INTFRAMESTART {DoFrame();} intframecont FRAMEEND {DoFrame();} framecontents
		|	
                ;

intframecont	:	frameprint {DoFrame();} intframecont
                |	FRAMETYPE intframecont
                |	INTFRAMESTART {DoFrame();} intframecont FRAMEEND {DoFrame();} intframecont
                |	
                ;

frameprint	: PARASTART | PGFTAG | PGFNUMSTRING | PARALINE |STRING | PARAEND
                | FNOTEUSE | FNOTESTART
                | FNOTEEND | XREFSTART | XREFSRCTEXT | XREFSRCFILE | XREFEND 
                | AFRAMEUSE | TABLEUSE | MARKERTYPE | MARKERTEXT | TEXTFLOWSTART | TEXTFLOWRECTID 
                | TEXTFLOWEND | textrect | TABLEROWSTART | TABLECELLSTART 
                | TABLECELLEND | TABLEROWEND | FFAMILY | FWEIGHT 
                | FANGLE | FUNDERLINE | FSUPSCRIPT | FSUBSCRIPT | SPECIALCHAR
                | FONTSTART | FONTEND | PGFFONTSTART | PGFFONTEND
                | variname
                ;

textrect	:	TEXTRECTIDUSE {TextRect();}
                ;

tables		:	TABLESSTART {TablesStart();} table TABLESEND {TablesEnd();}
                ; 

table		:	TABLESTART {TableStart();} tbl TABLEEND {TableEnd();} table
                |
                ;

tbl		:	tableid tbl
                | 	tabletitle tbl
                |	tablehead tbl
                |	tableformat tbl
                |	tablenumcolumns tbl
                |	tablebody tbl
                |	tablefoot tbl
                |
                ;


tableid		:	TABLEID {GetTableID();}
                ;

tableformat	:	TABLEFORMATSTART skips TABLEFORMATEND
                ;

tablenumcolumns	:	TABLENUMCOLUMNS {GetTableNumColumns();}
                ;

tabletitle	:	TABLETITLESTART {TableTitleStart();} paragraphs TABLETITLEEND {TableTitleEnd();} 
                ;

tablehead	:	TABLEHEADSTART {TableHeadStart();} tablerows TABLEHEADEND {TableHeadEnd();} 
                ;

tablefoot	:	TABLEFOOTSTART {TableFootStart();} tablerows TABLEFOOTEND {TableFootEnd();} 
                ;

tablebody	:	TABLEBODYSTART tablerows TABLEBODYEND 
                ;

tablerows	:	TABLEROWSTART {TableRowStart();} tablecells TABLEROWEND {TableRowEnd();} tablerows
                |
                ;

tablecells	:	TABLECELLSTART {TableCellStart();} paragraphs TABLECELLEND {TableCellEnd();} tablecells
                |
                ;

pages		:	PAGE {SetPageNum();} pagecont pages
                |
                ;

pagecont	:	STRING pagecont
                |	FONTSTART {FontStart();} fontcont FONTEND {FontEnd();} pagecont
                |	TEXTRECTIDUSE {PrintTextRectID();} pagecont
                |	SPECIALCHAR pagecont
                |	inlineframe pagecont
                |       variname pagecont
                |
                ;

inlineframe     :	INTFRAMESTART {InlineFrameStart();} framecontents FRAMEEND {InlineFrameEnd();}
                ;


textflow      	:	TEXTFLOWSTART {ChangeOutputFile();} paragraphs TEXTFLOWEND {EndTextFlow();} textflow
                |
                ;

fnote		:	FNOTESTART {NewFootnoteStart();} paragraphs FNOTEEND {NewFootnoteEnd();} 
                ;


paragraphs	:	paragraph paragraphs
                |	fnote paragraphs
                |
                ;

paragraph 	:	PARASTART {NewParagraph();} paratag fontinfo paranum paralines PARAEND {EndParagraph();} 
                ;

paratag		:	PGFTAG {SetPgfTag();}
                |
                ;

paranum		:	PGFNUMSTRING {ParagraphNumber();}
                |
                ;

paralines	:	PARALINE {HandleParaLine();} paracont paralines
                |
                ;

paracont	:	TEXTFLOWRECTID {PrintTextRectID();} paracont
                |	string paracont
                |
                ;

strings		:	string strings
                |
                ;

string		:	STRING {PrintString();} 
                |	FNOTEUSE {InsertFootnote();} 
                |	xref 
                |	AFRAMEUSE {AFrameUse();} 
                |	TABLEUSE {TableUse();} 
                |	MARKERTEXT {SetMarker();}
                |	MARKERTYPE {SetMarkerType();}
                | 	SPECIALCHAR {InsertSpecialChar();}
                | 	FONTSTART {FontStart();} fontcont FONTEND {FontEnd();}
                |       variname 
                ;

fontinfo	:	PGFFONTSTART fontskips PGFFONTEND
                |
                ;

fontskips	: 	fontskip fontskips
                |
                ;

fontskip	:	FFAMILY
                |	FWEIGHT
                |	FANGLE
                |	FUNDERLINE
                | 	FSUBSCRIPT
                | 	FSUPSCRIPT
                ;


fontcont	:	font fontcont
                |
                ;

font		:	fontfamily
                |	fontweight
                |	fontangle
                |	fontunderline
                | 	fontsubscript
                | 	fontsupscript
                ;

fontfamily	:	FFAMILY {ChangeFont();} 
                ;

fontweight	: 	FWEIGHT {ChangeFontWeight();} 
                ;

fontangle	:	FANGLE {ChangeFontAngle();} 
                ;

fontunderline	:	FUNDERLINE {ChangeFontUnderline();} 
                ;

fontsupscript	:	FSUPSCRIPT {ChangeFontSupscript();} 
                ;

fontsubscript	:	FSUBSCRIPT {ChangeFontSubscript();}
                ;
 
xref		: xrefstart xrefcont strings paralines xrefend
                ;

xrefstart	: XREFSTART {InsertReferenceStart();} 
                ;

xrefcont	: XREFSRCTEXT {SetReferenceNumber();} xrefcont 
                | XREFSRCFILE {SetReferenceFile();} xrefcont
                |
                ;

xrefend 	: XREFEND {InsertReferenceEnd();}
                ;

skips		: skip skips
                |
                ;

skip		: PARASTART | PGFTAG | PGFNUMSTRING | PARALINE | STRING | PARAEND
                | FNOTEUSE | FNOTESTART | SPECIALCHAR
                | FNOTEEND | XREFSTART | XREFSRCTEXT | XREFSRCFILE | XREFEND 
                | AFRAMEUSE | TABLEUSE | MARKERTEXT | MARKERTYPE | TEXTFLOWRECTID
                | TEXTRECTIDUSE | TABLEROWSTART | TABLECELLSTART 
                | TABLECELLEND | TABLEROWEND | FRAMESTART | FRAMEEND | FFAMILY 
                | FWEIGHT | FANGLE | FUNDERLINE | FSUPSCRIPT | FSUBSCRIPT
                | INTFRAMESTART | FRAMETYPE | FONTSTART | FONTEND | PGFFONTSTART | PGFFONTEND
                | variname
                ;

variformats     : VARIFORMSSTART variformnames VARIFORMSEND
                ;

variformnames   : VARIFORMSTART varinames VARIFORMEND variformnames
                |
                ;

varinames       : VARINAME {SaveVariable();} VARIDEF {SaveVariableDef();}
                ;

variname        : VARISTART VARINAME {NoteVariName();} VARIEND
                ;

%%

#include "trans.c"
#include "trans-code.c"
#include "trans_table-code.c"

/*
  * yyerror modified by
  * Jan van der Steen (jansteen@cwi.nl)
  */

yyerror(s)
char *s;
{
     if (s)
	 fprintf(stderr,"trans: ERROR in parsing. Got: \"%s\".\ntrans: Error: \"%s\"\n",
		 yytext, s);
     else
	fprintf(stderr, "trans: ERROR in parsing. Got %s .\n", yytext); 
}




